@@ -4,10 +4,11 @@ package Moose::Meta::Role::Method;
 use strict;
 use warnings;
 
-our $VERSION   = '0.54';
+our $VERSION   = '0.82';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
-use base 'Class::MOP::Method';
+use base 'Moose::Meta::Method';
 
 1;
 
@@ -21,13 +22,13 @@ Moose::Meta::Role::Method - A Moose Method metaclass for Roles
 
 =head1 DESCRIPTION
 
-This is primarily used to mark methods coming from a role 
+This is primarily used to mark methods coming from a role
 as being different. Right now it is nothing but a subclass
-of L<Class::MOP::Method>.
+of L<Moose::Meta::Method>.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no 
+All complex software has bugs lurking in it, and this module is no
 exception. If you find a bug please either email me, or add the bug
 to cpan-RT.
 
@@ -37,7 +38,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
