@@ -8,10 +8,13 @@ use metaclass;
 use overload '""'     => sub { shift->name },   # stringify to tc name
              fallback => 1;
 
-use Carp         'confess';
 use Scalar::Util qw(blessed refaddr);
+use Sub::Name qw(subname);
 
-our $VERSION   = '0.54';
+use base qw(Class::MOP::Object);
+
+our $VERSION   = '0.82';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 __PACKAGE__->meta->add_attribute('name'       => (reader => 'name'));
@@ -34,6 +37,7 @@ __PACKAGE__->meta->add_attribute('coercion'   => (
     accessor  => 'coercion',
     predicate => 'has_coercion'
 ));
+
 __PACKAGE__->meta->add_attribute('hand_optimized_type_constraint' => (
     init_arg  => 'optimized',
     accessor  => 'hand_optimized_type_constraint',
@@ -57,14 +61,39 @@ __PACKAGE__->meta->add_attribute('package_defined_in' => (
 
 sub new {
     my $class = shift;
-    my $self  = $class->meta->new_object(@_);
+    my ($first, @rest) = @_;
+    my %args = ref $first ? %$first : $first ? ($first, @rest) : ();
+    $args{name} = $args{name} ? "$args{name}" : "__ANON__";
+
+    my $self  = $class->_new(%args);
     $self->compile_type_constraint()
         unless $self->_has_compiled_type_constraint;
     return $self;
 }
 
-sub coerce   { ((shift)->coercion || confess "Cannot coerce without a type coercion")->coerce(@_) }
-sub check    { $_[0]->_compiled_type_constraint->($_[1]) ? 1 : undef }
+
+
+sub coerce {
+    my $self = shift;
+
+    my $coercion = $self->coercion;
+
+    unless ($coercion) {
+        require Moose;
+        Moose->throw_error("Cannot coerce without a type coercion");
+    }
+
+    return $_[0] if $self->check($_[0]);
+
+    return $coercion->coerce(@_);
+}
+
+sub check {
+    my ($self, @args) = @_;
+    my $constraint_subref = $self->_compiled_type_constraint;
+    return $constraint_subref->(@args) ? 1 : undef;
+}
+
 sub validate {
     my ($self, $value) = @_;
     if ($self->_compiled_type_constraint->($value)) {
@@ -77,14 +106,14 @@ sub validate {
 
 sub get_message {
     my ($self, $value) = @_;
-    $value = (defined $value ? overload::StrVal($value) : 'undef');
     if (my $msg = $self->message) {
         local $_ = $value;
         return $msg->($value);
     }
     else {
+        $value = (defined $value ? overload::StrVal($value) : 'undef');
         return "Validation failed for '" . $self->name . "' failed with value $value";
-    }    
+    }
 }
 
 ## type predicates ...
@@ -92,7 +121,7 @@ sub get_message {
 sub equals {
     my ( $self, $type_or_name ) = @_;
 
-    my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     return 1 if refaddr($self) == refaddr($other);
 
@@ -115,7 +144,7 @@ sub equals {
 sub is_a_type_of {
     my ($self, $type_or_name) = @_;
 
-    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     ($self->equals($type) || $self->is_subtype_of($type));
 }
@@ -123,7 +152,7 @@ sub is_a_type_of {
 sub is_subtype_of {
     my ($self, $type_or_name) = @_;
 
-    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
+    my $type = Moose::Util::TypeConstraints::find_type_constraint($type_or_name) or return;
 
     my $current = $self;
 
@@ -151,10 +180,12 @@ sub _actually_compile_type_constraint {
         if $self->has_hand_optimized_type_constraint;
 
     my $check = $self->constraint;
-    (defined $check)
-        || confess "Could not compile type constraint '"
+    unless ( defined $check ) {
+        require Moose;
+        Moose->throw_error( "Could not compile type constraint '"
                 . $self->name
-                . "' because no constraint check";
+                . "' because no constraint check" );
+    }
 
     return $self->_compile_subtype($check)
         if $self->has_parent;
@@ -167,7 +198,11 @@ sub _compile_hand_optimized_type_constraint {
 
     my $type_constraint = $self->hand_optimized_type_constraint;
 
-    confess unless ref $type_constraint;
+    unless ( ref $type_constraint ) {
+        require Moose;
+        Carp::confess ("Hand optimized type constraint for " . $self->name . " is not a code reference");
+        Moose->throw_error("Hand optimized type constraint is not a code reference");
+    }
 
     return $type_constraint;
 }
@@ -200,20 +235,22 @@ sub _compile_subtype {
         if ( $check == $null_constraint ) {
             return $optimized_parent;
         } else {
-            return Class::MOP::subname($self->name, sub {
+            return subname($self->name, sub {
                 return undef unless $optimized_parent->($_[0]);
-                local $_ = $_[0];
-                $check->($_[0]);
+                my (@args) = @_;
+                local $_ = $args[0];
+                $check->(@args);
             });
         }
     } else {
         # general case, check all the constraints, from the first parent to ourselves
         my @checks = @parents;
         push @checks, $check if $check != $null_constraint;
-        return Class::MOP::subname($self->name => sub {
-            local $_ = $_[0];
+        return subname($self->name => sub {
+            my (@args) = @_;
+            local $_ = $args[0];
             foreach my $check (@checks) {
-                return undef unless $check->($_[0]);
+                return undef unless $check->(@args);
             }
             return 1;
         });
@@ -225,9 +262,10 @@ sub _compile_type {
 
     return $check if $check == $null_constraint; # Item, Any
 
-    return Class::MOP::subname($self->name => sub {
-        local $_ = $_[0];
-        $check->($_[0]);
+    return subname($self->name => sub {
+        my (@args) = @_;
+        local $_ = $args[0];
+        $check->(@args);
     });
 }
 
@@ -244,9 +282,11 @@ sub _collect_all_parents {
     return @parents;
 }
 
-## this should get deprecated actually ...
-
-sub union { Carp::croak "DEPRECATED" }
+sub create_child_type {
+    my ($self, %opts) = @_;
+    my $class = ref $self;
+    return $class->new(%opts, parent => $self);
+}
 
 1;
 
@@ -260,84 +300,150 @@ Moose::Meta::TypeConstraint - The Moose Type Constraint metaclass
 
 =head1 DESCRIPTION
 
-For the most part, the only time you will ever encounter an
-instance of this class is if you are doing some serious deep
-introspection. This API should not be considered final, but
-it is B<highly unlikely> that this will matter to a regular
-Moose user.
+This class represents a single type constraint. Moose's built-in type
+constraints, as well as constraints you define, are all store in a
+L<Moose::Meta::TypeConstraint::Registry> object as objects of this
+class.
 
-If you wish to use features at this depth, please come to the
-#moose IRC channel on irc.perl.org and we can talk :)
+=head1 INHERITANCE
+
+C<Moose::Meta::TypeConstraint> is a subclass of L<Class::MOP::Object>.
 
 =head1 METHODS
 
 =over 4
 
-=item B<meta>
+=item B<< Moose::Meta::TypeConstraint->new(%options) >>
 
-=item B<new>
+This creates a new type constraint based on the provided C<%options>:
 
-=item B<equals ($type_name_or_object)>
+=over 8
 
-=item B<is_a_type_of ($type_name_or_object)>
+=item * name
 
-This checks the current type name, and if it does not match,
-checks if it is a subtype of it.
+The constraint name. If a name is not provided, it will be set to
+"__ANON__".
 
-=item B<is_subtype_of ($type_name_or_object)>
+=item * parent
 
-=item B<compile_type_constraint>
+A C<Moose::Meta::TypeConstraint> object which is the parent type for
+the type being created. This is optional.
 
-=item B<coerce ($value)>
+=item * constraint
 
-This will apply the type-coercion if applicable.
+This is the subroutine reference that implements the actual constraint
+check. This defaults to a subroutine which always returns true.
 
-=item B<check ($value)>
+=item * message
 
-This method will return a true (C<1>) if the C<$value> passes the
-constraint, and false (C<0>) otherwise.
+A subroutine reference which is used to generate an error message when
+the constraint fails. This is optional.
 
-=item B<validate ($value)>
+=item * coercion
 
-This method is similar to C<check>, but it deals with the error
-message. If the C<$value> passes the constraint, C<undef> will be
-returned. If the C<$value> does B<not> pass the constraint, then
-the C<message> will be used to construct a custom error message.
+A L<Moose::Meta::TypeCoercion> object representing the coercions to
+the type. This is optional.
 
-=item B<name>
+=item * optimized
 
-=item B<parent>
-
-=item B<has_parent>
-
-=item B<parents>
-
-=item B<constraint>
-
-=item B<has_message>
-
-=item B<message>
-
-=item B<get_message ($value)>
-
-=item B<has_coercion>
-
-=item B<coercion>
-
-=item B<hand_optimized_type_constraint>
-
-=item B<has_hand_optimized_type_constraint>
+This is a variant of the C<constraint> parameter that is somehow
+optimized. Typically, this means incorporating both the type's
+constraint and all of its parents' constraints into a single
+subroutine reference.
 
 =back
 
-=head2 DEPRECATED METHOD
+=item B<< $constraint->equals($type_name_or_object) >>
 
-=over 4
+Returns true if the supplied name or type object is the same as the
+current type.
 
-=item B<union>
+=item B<< $constraint->is_subtype_of($type_name_or_object) >>
 
-This was just bad idea on my part,.. use the L<Moose::Meta::TypeConstraint::Union>
-itself instead.
+Returns true if the supplied name or type object is a parent of the
+current type.
+
+=item B<< $constraint->is_a_type_of($type_name_or_object) >>
+
+Returns true if the given type is the same as the current type, or is
+a parent of the current type. This is a shortcut for checking
+C<equals> and C<is_subtype_of>.
+
+=item B<< $constraint->coerce($value) >>
+
+This will attempt to coerce the value to the type. If the type does
+have any defined coercions this will throw an error.
+
+=item B<< $constraint->check($value) >>
+
+Returns true if the given value passes the constraint for the type.
+
+=item B<< $constraint->validate($value) >>
+
+This is similar to C<check>. However, if the type I<is valid> then the
+method returns an explicit C<undef>. If the type is not valid, we call
+C<< $self->get_message($value) >> internally to generate an error
+message.
+
+=item B<< $constraint->name >>
+
+Returns the type's name, as provided to the constructor.
+
+=item B<< $constraint->parent >>
+
+Returns the type's parent, as provided to the constructor, if any.
+
+=item B<< $constraint->has_parent >>
+
+Returns true if the type has a parent type.
+
+=item B<< $constraint->parents >>
+
+A synonym for C<parent>. This is useful for polymorphism with types
+that can have more than one parent.
+
+=item B<< $constraint->constraint >>
+
+Returns the type's constraint, as provided to the constructor.
+
+=item B<< $constraint->get_message($value) >>
+
+This generates a method for the given value. If the type does not have
+an explicit message, we generate a default message.
+
+=item B<< $constraint->has_message >>
+
+Returns true if the type has a message.
+
+=item B<< $constraint->message >>
+
+Returns the type's message as a subroutine reference.
+
+=item B<< $constraint->coercion >>
+
+Returns the type's L<Moose::Meta::TypeCoercion> object, if one
+exists.
+
+=item B<< $constraint->has_coercion >>
+
+Returns true if the type has a coercion.
+
+=item B<< $constraint->hand_optimized_type_constraint >>
+
+Returns the type's hand optimized constraint, as provided to the
+constructor via the C<optimized> option.
+
+=item B<< $constraint->has_hand_optimized_type_constraint >>
+
+Returns true if the type has an optimized constraint.
+
+=item B<< $constraint->create_child_type(%options) >>
+
+This returns a new type constraint of the same class using the
+provided C<%options>. The C<parent> option will be the current type.
+
+This method exists so that subclasses of this class can override this
+behavior and change how child types are created.
 
 =back
 
@@ -353,7 +459,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
