@@ -8,7 +8,7 @@ use Scalar::Util 'blessed';
 use Moose::Util::TypeConstraints;
 use Moose::Meta::TypeConstraint::Parameterizable;
 
-our $VERSION   = '0.63';
+our $VERSION   = '0.82';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -25,7 +25,7 @@ sub equals {
     my $other = Moose::Util::TypeConstraints::find_type_constraint($type_or_name);
 
     return unless $other->isa(__PACKAGE__);
-    
+
     return (
         $self->type_parameter->equals( $other->type_parameter )
             and
@@ -35,25 +35,30 @@ sub equals {
 
 sub compile_type_constraint {
     my $self = shift;
-    
-    ($self->has_type_parameter)
-        || Moose->throw_error("You cannot create a Higher Order type without a type parameter");
-        
+
+    unless ( $self->has_type_parameter ) {
+        require Moose;
+        Moose->throw_error("You cannot create a Higher Order type without a type parameter");
+    }
+
     my $type_parameter = $self->type_parameter;
-    
-    (blessed $type_parameter && $type_parameter->isa('Moose::Meta::TypeConstraint'))
-        || Moose->throw_error("The type parameter must be a Moose meta type");
+
+    unless ( blessed $type_parameter && $type_parameter->isa('Moose::Meta::TypeConstraint') ) {
+        require Moose;
+        Moose->throw_error("The type parameter must be a Moose meta type");
+    }
 
     foreach my $type (Moose::Util::TypeConstraints::get_all_parameterizable_types()) {
         if (my $constraint = $type->generate_constraint_for($self)) {
             $self->_set_constraint($constraint);
-            return $self->SUPER::compile_type_constraint;            
+            return $self->SUPER::compile_type_constraint;
         }
     }
-    
-    # if we get here, then we couldn't 
+
+    # if we get here, then we couldn't
     # find a way to parameterize this type
-    Moose->throw_error("The " . $self->name . " constraint cannot be used, because " 
+    require Moose;
+    Moose->throw_error("The " . $self->name . " constraint cannot be used, because "
           . $self->parent->name . " doesn't subtype or coerce from a parameterizable type.");
 }
 
@@ -71,29 +76,21 @@ __END__
 
 =head1 NAME
 
-Moose::Meta::TypeConstraint::Parameterized - Higher Order type constraints for Moose
+Moose::Meta::TypeConstraint::Parameterized - Type constraints with a bound parameter (ArrayRef[Int])
 
 =head1 METHODS
 
-=over 4
+This class is intentionally not documented because the API is
+confusing and needs some work.
 
-=item B<compile_type_constraint>
+=head1 INHERITANCE
 
-=item B<type_parameter>
-
-=item B<has_type_parameter>
-
-=item B<meta>
-
-=item B<equals>
-
-=item B<create_child_type>
-
-=back
+C<Moose::Meta::TypeConstraint::Parameterized> is a subclass of
+L<Moose::Meta::TypeConstraint>.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no 
+All complex software has bugs lurking in it, and this module is no
 exception. If you find a bug please either email me, or add the bug
 to cpan-RT.
 
@@ -103,7 +100,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
